@@ -15,19 +15,20 @@
 # limitations under the License.
 
 for lib in \
-        os-client-config \
-        keystoneauth \
         python-novaclient \
         python-keystoneclient \
-        python-cinderclient \
-        python-neutronclient \
         python-ironicclient \
-        python-heatclient \
         python-designateclient \
-        python-magnumclient
+        os-client-config \
+        keystoneauth
 do
     egg=$(echo $lib | tr '-' '_' | sed 's/python-//')
     if [ -d /opt/stack/new/$lib ] ; then
-        pip install -q -U -e "git+file:///opt/stack/new/$lib#egg=$egg"
+        tip_location="git+file:///opt/stack/new/$lib#egg=$egg"
+        echo "$(which pip) install -U -e $tip_location"
+        pip uninstall -y $lib
+        pip install -U -e $tip_location
+    else
+        echo "$lib not found in /opt/stack/new/$lib"
     fi
 done
