@@ -39,7 +39,6 @@ _SERVER_FIELDS = (
     'private_v4',
     'public_v4',
     'public_v6',
-    'security_groups',
     'status',
     'updated',
     'user_id',
@@ -142,6 +141,15 @@ class Normalizer(object):
 
         return new_limits
 
+    def _remove_novaclient_artifacts(self, item):
+        # Remove novaclient artifacts
+        item.pop('links', None)
+        item.pop('NAME_ATTR', None)
+        item.pop('HUMAN_ID', None)
+        item.pop('human_id', None)
+        item.pop('request_ids', None)
+        item.pop('x_openstack_request_ids', None)
+
     def _normalize_flavors(self, flavors):
         """ Normalize a list of flavor objects """
         ret = []
@@ -157,11 +165,8 @@ class Normalizer(object):
         flavor = flavor.copy()
 
         # Discard noise
+        self._remove_novaclient_artifacts(flavor)
         flavor.pop('links', None)
-        flavor.pop('NAME_ATTR', None)
-        flavor.pop('HUMAN_ID', None)
-        flavor.pop('human_id', None)
-        flavor.pop('request_ids', None)
 
         ephemeral = int(_pop_or_get(
             flavor, 'OS-FLV-EXT-DATA:ephemeral', 0, self.strict_mode))
@@ -211,10 +216,8 @@ class Normalizer(object):
         # This copy is to keep things from getting epically weird in tests
         image = image.copy()
 
-        image.pop('links', None)
-        image.pop('NAME_ATTR', None)
-        image.pop('HUMAN_ID', None)
-        image.pop('human_id', None)
+        # Discard noise
+        self._remove_novaclient_artifacts(image)
 
         properties = image.pop('properties', {})
         visibility = image.pop('visibility', None)
@@ -297,6 +300,9 @@ class Normalizer(object):
         # Copy incoming group because of shared dicts in unittests
         group = group.copy()
 
+        # Discard noise
+        self._remove_novaclient_artifacts(group)
+
         rules = self._normalize_secgroup_rules(
             group.pop('security_group_rules', group.pop('rules', [])))
         project_id = group.pop('tenant_id', '')
@@ -387,10 +393,7 @@ class Normalizer(object):
         # Copy incoming server because of shared dicts in unittests
         server = server.copy()
 
-        server.pop('links', None)
-        server.pop('NAME_ATTR', None)
-        server.pop('HUMAN_ID', None)
-        server.pop('human_id', None)
+        self._remove_novaclient_artifacts(server)
 
         ret['id'] = server.pop('id')
         ret['name'] = server.pop('name')
@@ -437,6 +440,9 @@ class Normalizer(object):
             short_key = key.split(':')[1]
             ret[short_key] = _pop_or_get(server, key, None, self.strict_mode)
 
+        # Protect against security_groups being None
+        ret['security_groups'] = server.pop('security_groups', None) or []
+
         for field in _SERVER_FIELDS:
             ret[field] = server.pop(field, None)
         ret['interface_ip'] = ''
@@ -570,10 +576,7 @@ class Normalizer(object):
         project = project.copy()
 
         # Discard noise
-        project.pop('links', None)
-        project.pop('NAME_ATTR', None)
-        project.pop('HUMAN_ID', None)
-        project.pop('human_id', None)
+        self._remove_novaclient_artifacts(project)
 
         # In both v2 and v3
         project_id = project.pop('id')
@@ -676,6 +679,11 @@ class Normalizer(object):
         :returns: A list of normalized dicts.
         """
         ret = []
+        # With pagination we don't get a top-level container
+        # so we need to explicitly extract the list of volumes
+        if isinstance(volumes, dict):
+            volumes = volumes.get('volumes', [])
+
         for volume in volumes:
             ret.append(self._normalize_volume(volume))
         return ret
@@ -685,10 +693,7 @@ class Normalizer(object):
         volume = volume.copy()
 
         # Discard noise
-        volume.pop('links', None)
-        volume.pop('NAME_ATTR', None)
-        volume.pop('HUMAN_ID', None)
-        volume.pop('human_id', None)
+        self._remove_novaclient_artifacts(volume)
 
         volume_id = volume.pop('id')
 
@@ -762,17 +767,36 @@ class Normalizer(object):
                 ret.setdefault(key, val)
         return ret
 
+    def _normalize_volume_attachment(self, attachment):
+        """ Normalize a volume attachment object"""
+
+        attachment = attachment.copy()
+
+        # Discard noise
+        self._remove_novaclient_artifacts(attachment)
+        return munch.Munch(**attachment)
+
+    def _normalize_volume_backups(self, backups):
+        ret = []
+        for backup in backups:
+            ret.append(self._normalize_volume_backup(backup))
+        return ret
+
+    def _normalize_volume_backup(self, backup):
+        """ Normalize a valume backup object"""
+
+        backup = backup.copy()
+        # Discard noise
+        self._remove_novaclient_artifacts(backup)
+        return munch.Munch(**backup)
+
     def _normalize_compute_usage(self, usage):
         """ Normalize a compute usage object """
 
         usage = usage.copy()
 
         # Discard noise
-        usage.pop('links', None)
-        usage.pop('NAME_ATTR', None)
-        usage.pop('HUMAN_ID', None)
-        usage.pop('human_id', None)
-        usage.pop('request_ids', None)
+        self._remove_novaclient_artifacts(usage)
         project_id = usage.pop('tenant_id', None)
 
         ret = munch.Munch(
@@ -797,8 +821,8 @@ class Normalizer(object):
                 'total_server_groups_used',
                 'total_vcpus_usage'):
             ret[key] = usage.pop(key, 0)
-        ret['started_at'] = usage.pop('start')
-        ret['stopped_at'] = usage.pop('stop')
+        ret['started_at'] = usage.pop('start', None)
+        ret['stopped_at'] = usage.pop('stop', None)
         ret['server_usages'] = self._normalize_server_usages(
             usage.pop('server_usages', []))
         ret['properties'] = usage
@@ -949,10 +973,8 @@ class Normalizer(object):
         stack = stack.copy()
 
         # Discard noise
-        stack.pop('HUMAN_ID', None)
-        stack.pop('human_id', None)
-        stack.pop('NAME_ATTR', None)
-        stack.pop('links', None)
+        self._remove_novaclient_artifacts(stack)
+
         # Discard things heatclient adds that aren't in the REST
         stack.pop('action', None)
         stack.pop('status', None)
@@ -995,3 +1017,21 @@ class Normalizer(object):
             name=ret['name'], id=ret['id'])
         ret['properties'] = stack
         return ret
+
+    def _normalize_machines(self, machines):
+        """Normalize Ironic Machines"""
+        ret = []
+        for machine in machines:
+            ret.append(self._normalize_machine(machine))
+        return ret
+
+    def _normalize_machine(self, machine):
+        """Normalize Ironic Machine"""
+        machine = machine.copy()
+
+        # Discard noise
+        self._remove_novaclient_artifacts(machine)
+
+        # TODO(mordred) Normalize this resource
+
+        return machine
