@@ -25,8 +25,6 @@ import sys
 import time
 
 from decorator import decorator
-from heatclient import exc as heat_exc
-from neutronclient.common import exceptions as neutron_exc
 from novaclient import exceptions as nova_exc
 
 from shade import _log
@@ -97,6 +95,16 @@ def _make_unicode(input):
         return str(input)
 
 
+def _dictify_resource(resource):
+    if isinstance(resource, list):
+        return [_dictify_resource(r) for r in resource]
+    else:
+        if hasattr(resource, 'toDict'):
+            return resource.toDict()
+        else:
+            return resource
+
+
 def _filter_list(data, name_or_id, filters):
     """Filter a list by name/ID and arbitrary meta data.
 
@@ -369,20 +377,6 @@ def normalize_flavor_accesses(flavor_accesses):
     ]
 
 
-def normalize_stacks(stacks):
-    """ Normalize Stack Object """
-    for stack in stacks:
-        stack['name'] = stack['stack_name']
-    return stacks
-
-
-def normalize_cluster_templates(cluster_templates):
-    """Normalize Magnum cluster_templates."""
-    for cluster_template in cluster_templates:
-        cluster_template['id'] = cluster_template['uuid']
-    return cluster_templates
-
-
 def valid_kwargs(*valid_args):
     # This decorator checks if argument passed as **kwargs to a function are
     # present in valid_args.
@@ -432,37 +426,6 @@ def cache_on_arguments(*cache_on_args, **cache_on_kwargs):
     return _inner_cache_on_arguments
 
 
-@contextlib.contextmanager
-def heat_exceptions(error_message):
-    try:
-        yield
-    except heat_exc.NotFound as e:
-        raise exc.OpenStackCloudResourceNotFound(
-            "{msg}: {exc}".format(msg=error_message, exc=str(e)))
-    except Exception as e:
-        raise exc.OpenStackCloudException(
-            "{msg}: {exc}".format(msg=error_message, exc=str(e)))
-
-
-@contextlib.contextmanager
-def neutron_exceptions(error_message):
-    try:
-        yield
-    except neutron_exc.NotFound as e:
-        raise exc.OpenStackCloudResourceNotFound(
-            "{msg}: {exc}".format(msg=error_message, exc=str(e)))
-    except neutron_exc.NeutronClientException as e:
-        if e.status_code == 404:
-            raise exc.OpenStackCloudURINotFound(
-                "{msg}: {exc}".format(msg=error_message, exc=str(e)))
-        else:
-            raise exc.OpenStackCloudException(
-                "{msg}: {exc}".format(msg=error_message, exc=str(e)))
-    except Exception as e:
-        raise exc.OpenStackCloudException(
-            "{msg}: {exc}".format(msg=error_message, exc=str(e)))
-
-
 @contextlib.contextmanager
 def shade_exceptions(error_message=None):
     """Context manager for dealing with shade exceptions.
@@ -671,7 +634,7 @@ def generate_patches_from_kwargs(operation, **kwargs):
                  'value': v,
                  'path': '/%s' % k}
         patches.append(patch)
-    return patches
+    return sorted(patches)
 
 
 class FileSegment(object):
