@@ -25,8 +25,8 @@ from shade import exc
 NON_CALLABLES = (six.string_types, bool, dict, int, float, list, type(None))
 
 
-def find_nova_addresses(addresses, ext_tag=None, key_name=None, version=4):
-
+def find_nova_interfaces(addresses, ext_tag=None, key_name=None, version=4,
+                         mac_addr=None):
     ret = []
     for (k, v) in iter(addresses.items()):
         if key_name is not None and k != key_name:
@@ -48,12 +48,32 @@ def find_nova_addresses(addresses, ext_tag=None, key_name=None, version=4):
                     # Type doesn't match, continue with next one
                     continue
 
-            if interface_spec['version'] == version:
-                ret.append(interface_spec['addr'])
+            if mac_addr is not None:
+                if 'OS-EXT-IPS-MAC:mac_addr' not in interface_spec:
+                    # mac_addr is specified, but this interface has no mac_addr
+                    # We could actually return right away as this means that
+                    # this cloud doesn't support OS-EXT-IPS-MAC. Nevertheless,
+                    # it would be better to perform an explicit check. e.g.:
+                    #   cloud._has_nova_extension('OS-EXT-IPS-MAC')
+                    # But this needs cloud to be passed to this function.
+                    continue
+                elif interface_spec['OS-EXT-IPS-MAC:mac_addr'] != mac_addr:
+                    # MAC doesn't match, continue with next one
+                    continue
 
+            if interface_spec['version'] == version:
+                ret.append(interface_spec)
     return ret
 
 
+def find_nova_addresses(addresses, ext_tag=None, key_name=None, version=4,
+                        mac_addr=None):
+    interfaces = find_nova_interfaces(addresses, ext_tag, key_name, version,
+                                      mac_addr)
+    addrs = [i['addr'] for i in interfaces]
+    return addrs
+
+
 def get_server_ip(server, public=False, cloud_public=True, **kwargs):
     """Get an IP from the Nova addresses dict
 
@@ -89,6 +109,13 @@ def get_server_private_ip(server, cloud=None):
     if cloud and not cloud.use_internal_network():
         return None
 
+    # Try to get a floating IP interface. If we have one then return the
+    # private IP address associated with that floating IP for consistency.
+    fip_ints = find_nova_interfaces(server['addresses'], ext_tag='floating')
+    fip_mac = None
+    if fip_ints:
+        fip_mac = fip_ints[0].get('OS-EXT-IPS-MAC:mac_addr')
+
     # Short circuit the ports/networks search below with a heavily cached
     # and possibly pre-configured network name
     if cloud:
@@ -96,12 +123,13 @@ def get_server_private_ip(server, cloud=None):
         for int_net in int_nets:
             int_ip = get_server_ip(
                 server, key_name=int_net['name'],
-                cloud_public=not cloud.private)
+                cloud_public=not cloud.private,
+                mac_addr=fip_mac)
             if int_ip is not None:
                 return int_ip
 
     ip = get_server_ip(
-        server, ext_tag='fixed', key_name='private')
+        server, ext_tag='fixed', key_name='private', mac_addr=fip_mac)
     if ip:
         return ip
 
@@ -344,7 +372,8 @@ def _get_supplemental_addresses(cloud, server):
     try:
         # Don't bother doing this before the server is active, it's a waste
         # of an API call while polling for a server to come up
-        if cloud._has_floating_ips() and server['status'] == 'ACTIVE':
+        if (cloud.has_service('network') and cloud._has_floating_ips() and
+                server['status'] == 'ACTIVE'):
             for port in cloud.search_ports(
                     filters=dict(device_id=server['id'])):
                 for fip in cloud.search_floating_ips(
@@ -406,7 +435,7 @@ def expand_server_security_groups(cloud, server):
         groups = cloud.list_server_security_groups(server)
     except exc.OpenStackCloudException:
         groups = []
-    server['security_groups'] = groups
+    server['security_groups'] = groups or []
 
 
 def get_hostvars_from_server(cloud, server, mounts=None):
@@ -458,15 +487,6 @@ def get_hostvars_from_server(cloud, server, mounts=None):
 
 
 def _log_request_id(obj, request_id):
-    # Add it, if passed in, even though we're going to pop in a second,
-    # just to make the logic simpler
-    if request_id is not None:
-        obj['x_openstack_request_ids'] = [request_id]
-
-    request_id = None
-    request_ids = obj.pop('x_openstack_request_ids', None)
-    if request_ids:
-        request_id = request_ids[0]
     if request_id:
         # Log the request id and object id in a specific logger. This way
         # someone can turn it on if they're interested in this kind of tracing.
@@ -485,7 +505,7 @@ def _log_request_id(obj, request_id):
     return obj
 
 
-def obj_to_dict(obj, request_id=None):
+def obj_to_dict(obj):
     """ Turn an object with attributes into a dict suitable for serializing.
 
     Some of the things that are returned in OpenStack are objects with
@@ -522,26 +542,17 @@ def obj_to_dict(obj, request_id=None):
             continue
         if isinstance(value, NON_CALLABLES) and not key.startswith('_'):
             instance[key] = value
-    return _log_request_id(instance, request_id)
+    return instance
 
 
-def obj_list_to_dict(obj_list, request_id=None):
+def obj_list_to_dict(obj_list):
     """Enumerate through lists of objects and return lists of dictonaries.
 
     Some of the objects returned in OpenStack are actually lists of objects,
     and in order to expose the data structures as JSON, we need to facilitate
     the conversion to lists of dictonaries.
     """
-    new_list = []
-    if not request_id:
-        request_id = getattr(obj_list, 'request_ids', [None])[0]
-    if request_id:
-        log = _log.setup_logging('shade.request_ids')
-        log.debug("Retrieved a list. Request ID %(request_id)s",
-                  {'request_id': request_id})
-    for obj in obj_list:
-        new_list.append(obj_to_dict(obj))
-    return new_list
+    return [obj_to_dict(obj) for obj in obj_list]
 
 
 def warlock_to_dict(obj):
