@@ -11,12 +11,11 @@
 # limitations under the License.
 
 import datetime
+import iso8601
 import jsonpatch
 
-from ironicclient import client as ironic_client
 from ironicclient import exceptions as ironic_exceptions
 from novaclient import exceptions as nova_exceptions
-from cinderclient import exceptions as cinder_exceptions
 
 from shade.exc import *  # noqa
 from shade import openstackcloud
@@ -33,30 +32,6 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
     See the :class:`OpenStackCloud` class for a description of most options.
     """
 
-    def __init__(self, *args, **kwargs):
-        super(OperatorCloud, self).__init__(*args, **kwargs)
-        self._ironic_client = None
-
-    # Set the ironic API microversion to a known-good
-    # supported/tested with the contents of shade.
-    #
-    # Note(TheJulia): Defaulted to version 1.6 as the ironic
-    # state machine changes which will increment the version
-    # and break an automatic transition of an enrolled node
-    # to an available state. Locking the version is intended
-    # to utilize the original transition until shade supports
-    # calling for node inspection to allow the transition to
-    # take place automatically.
-    ironic_api_microversion = '1.6'
-
-    @property
-    def ironic_client(self):
-        if self._ironic_client is None:
-            self._ironic_client = self._get_client(
-                'baremetal', ironic_client.Client,
-                os_ironic_api_version=self.ironic_api_microversion)
-        return self._ironic_client
-
     def list_nics(self):
         with _utils.shade_exceptions("Error fetching machine port list"):
             return self.manager.submit_task(_tasks.MachinePortList())
@@ -76,7 +51,8 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             return None
 
     def list_machines(self):
-        return self.manager.submit_task(_tasks.MachineNodeList())
+        return self._normalize_machines(
+            self.manager.submit_task(_tasks.MachineNodeList()))
 
     def get_machine(self, name_or_id):
         """Get Machine by name or uuid
@@ -90,8 +66,9 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
                   nodes are found.
         """
         try:
-            return self.manager.submit_task(
-                _tasks.MachineNodeGet(node_id=name_or_id))
+            return self._normalize_machine(
+                self.manager.submit_task(
+                    _tasks.MachineNodeGet(node_id=name_or_id)))
         except ironic_exceptions.ClientException:
             return None
 
@@ -776,6 +753,8 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         type_ = kwargs.pop('type', None)
         service_type = kwargs.pop('service_type', None)
 
+        # TODO(mordred) When this changes to REST, force interface=admin
+        # in the adapter call
         if self.cloud_config.get_api_version('identity').startswith('2'):
             kwargs['service_type'] = type_ or service_type
         else:
@@ -800,6 +779,8 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
                 'Unavailable Feature: Service update requires Identity v3'
             )
 
+        # TODO(mordred) When this changes to REST, force interface=admin
+        # in the adapter call
         # NOTE(SamYaple): Keystone v3 only accepts 'type' but shade accepts
         #                 both 'type' and 'service_type' with a preference
         #                 towards 'type'
@@ -825,6 +806,8 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         :raises: ``OpenStackCloudException`` if something goes wrong during the
             openstack API call.
         """
+        # TODO(mordred) When this changes to REST, force interface=admin
+        # in the adapter call
         with _utils.shade_exceptions():
             services = self.manager.submit_task(_tasks.ServiceList())
         return _utils.normalize_keystone_services(services)
@@ -882,6 +865,8 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             service_kwargs = {'id': service['id']}
         else:
             service_kwargs = {'service': service['id']}
+        # TODO(mordred) When this changes to REST, force interface=admin
+        # in the adapter call
         with _utils.shade_exceptions("Failed to delete service {id}".format(
                 id=service['id'])):
             self.manager.submit_task(_tasks.ServiceDelete(**service_kwargs))
@@ -971,6 +956,8 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             if region is not None:
                 kwargs['region'] = region
 
+        # TODO(mordred) When this changes to REST, force interface=admin
+        # in the adapter call
         with _utils.shade_exceptions(
             "Failed to create endpoint for service"
             " {service}".format(service=service['name'])
@@ -997,6 +984,8 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         if service_name_or_id is not None:
             kwargs['service'] = service_name_or_id
 
+        # TODO(mordred) When this changes to REST, force interface=admin
+        # in the adapter call
         with _utils.shade_exceptions(
             "Failed to update endpoint {}".format(endpoint_id)
         ):
@@ -1016,6 +1005,8 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         # the keystone api, but since the return of all the endpoints even in
         # large environments is small, we can continue to filter in shade just
         # like the v2 api.
+        # TODO(mordred) When this changes to REST, force interface=admin
+        # in the adapter call
         with _utils.shade_exceptions("Failed to list endpoints"):
             endpoints = self.manager.submit_task(_tasks.EndpointList())
 
@@ -1074,6 +1065,8 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             self.log.debug("Endpoint %s not found for deleting", id)
             return False
 
+        # TODO(mordred) When this changes to REST, force interface=admin
+        # in the adapter call
         if self.cloud_config.get_api_version('identity').startswith('2'):
             endpoint_kwargs = {'id': endpoint['id']}
         else:
@@ -1795,8 +1788,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
                 data['tenant'] = data.pop('project')
                 self.manager.submit_task(_tasks.RoleRemoveUser(**data))
             else:
-                if data.get('project') is None \
-                        and data.get('domain') is None:
+                if data.get('project') is None and data.get('domain') is None:
                     raise OpenStackCloudException(
                         'Must specify either a domain or project')
                 self.manager.submit_task(_tasks.RoleRevokeUser(**data))
@@ -1999,9 +1991,9 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
                 "Unable to get volume type access {name}".format(
                     name=name_or_id)):
             return self._normalize_volume_type_accesses(
-                self.manager.submit_task(
-                    _tasks.VolumeTypeAccessList(volume_type=volume_type))
-            )
+                self._volume_client.get(
+                    '/types/{id}/os-volume-type-access'.format(
+                        id=volume_type.id)))
 
     def add_volume_type_access(self, name_or_id, project_id):
         """Grant access on a volume_type to a project.
@@ -2017,14 +2009,14 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         if not volume_type:
             raise OpenStackCloudException(
                 "VolumeType not found: %s" % name_or_id)
-        with _utils.shade_exceptions(
-                "Unable to authorize {project} "
-                "to use volume type {name}".format(
-                    name=name_or_id, project=project_id
-                )):
-            self.manager.submit_task(
-                _tasks.VolumeTypeAccessAdd(
-                    volume_type=volume_type, project=project_id))
+        with _utils.shade_exceptions():
+            payload = {'project': project_id}
+            self._volume_client.post(
+                '/types/{id}/action'.format(id=volume_type.id),
+                json=dict(addProjectAccess=payload),
+                error_message="Unable to authorize {project} "
+                              "to use volume type {name}".format(
+                              name=name_or_id, project=project_id))
 
     def remove_volume_type_access(self, name_or_id, project_id):
         """Revoke access on a volume_type to a project.
@@ -2038,14 +2030,14 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         if not volume_type:
             raise OpenStackCloudException(
                 "VolumeType not found: %s" % name_or_id)
-        with _utils.shade_exceptions(
-                "Unable to revoke {project} "
-                "to use volume type {name}".format(
-                    name=name_or_id, project=project_id
-                )):
-            self.manager.submit_task(
-                _tasks.VolumeTypeAccessRemove(
-                    volume_type=volume_type, project=project_id))
+        with _utils.shade_exceptions():
+            payload = {'project': project_id}
+            self._volume_client.post(
+                '/types/{id}/action'.format(id=volume_type.id),
+                json=dict(removeProjectAccess=payload),
+                error_message="Unable to revoke {project} "
+                              "to use volume type {name}".format(
+                              name=name_or_id, project=project_id))
 
     def set_compute_quotas(self, name_or_id, **kwargs):
         """ Set a quota in a project
@@ -2112,22 +2104,60 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         except nova_exceptions.BadRequest:
             raise OpenStackCloudException("nova client call failed")
 
-    def get_compute_usage(self, name_or_id, start, end=None):
+    def get_compute_usage(self, name_or_id, start=None, end=None):
         """ Get usage for a specific project
 
         :param name_or_id: project name or id
-        :param start: :class:`datetime.datetime` Start date in UTC
-        :param end: :class:`datetime.datetime` End date in UTC. Defaults to now
+        :param start: :class:`datetime.datetime` or string. Start date in UTC
+                      Defaults to 2010-07-06T12:00:00Z (the date the OpenStack
+                      project was started)
+        :param end: :class:`datetime.datetime` or string. End date in UTC.
+                    Defaults to now
         :raises: OpenStackCloudException if it's not a valid project
 
         :returns: Munch object with the usage
         """
+        def parse_date(date):
+            try:
+                return iso8601.parse_date(date)
+            except iso8601.iso8601.ParseError:
+                # Yes. This is an exception mask. However,iso8601 is an
+                # implementation detail - and the error message is actually
+                # less informative.
+                raise OpenStackCloudException(
+                    "Date given, {date}, is invalid. Please pass in a date"
+                    " string in ISO 8601 format -"
+                    " YYYY-MM-DDTHH:MM:SS".format(
+                        date=date))
+
+        def parse_datetime_for_nova(date):
+            # Must strip tzinfo from the date- it breaks Nova. Also,
+            # Nova is expecting this in UTC. If someone passes in an
+            # ISO8601 date string or a datetime with timzeone data attached,
+            # strip the timezone data but apply offset math first so that
+            # the user's well formed perfectly valid date will be used
+            # correctly.
+            offset = date.utcoffset()
+            if offset:
+                date = date - datetime.timedelta(hours=offset)
+            return date.replace(tzinfo=None)
+
+        if not start:
+            start = parse_date('2010-07-06')
+        elif not isinstance(start, datetime.datetime):
+            start = parse_date(start)
+        if not end:
+            end = datetime.datetime.utcnow()
+        elif not isinstance(start, datetime.datetime):
+            end = parse_date(end)
+
+        start = parse_datetime_for_nova(start)
+        end = parse_datetime_for_nova(end)
+
         proj = self.get_project(name_or_id)
         if not proj:
             raise OpenStackCloudException("project does not exist: {}".format(
                 name=proj.id))
-        if not end:
-            end = datetime.datetime.now()
 
         with _utils.shade_exceptions(
                 "Unable to get resources usage for project: {name}".format(
@@ -2151,12 +2181,11 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         if not proj:
             raise OpenStackCloudException("project does not exist")
 
-        try:
-            self.manager.submit_task(
-                _tasks.CinderQuotasSet(tenant_id=proj.id,
-                                       **kwargs))
-        except cinder_exceptions.BadRequest:
-            raise OpenStackCloudException("No valid quota or resource")
+        kwargs['tenant_id'] = proj.id
+        self._volume_client.put(
+            '/os-quota-sets/{tenant_id}'.format(tenant_id=proj.id),
+            json={'quota_set': kwargs},
+            error_message="No valid quota or resource")
 
     def get_volume_quotas(self, name_or_id):
         """ Get volume quotas for a project
@@ -2169,11 +2198,10 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         proj = self.get_project(name_or_id)
         if not proj:
             raise OpenStackCloudException("project does not exist")
-        try:
-            return self.manager.submit_task(
-                _tasks.CinderQuotasGet(tenant_id=proj.id))
-        except cinder_exceptions.BadRequest:
-            raise OpenStackCloudException("cinder client call failed")
+
+        return self._volume_client.get(
+            '/os-quota-sets/{tenant_id}'.format(tenant_id=proj.id),
+            error_message="cinder client call failed")
 
     def delete_volume_quotas(self, name_or_id):
         """ Delete volume quotas for a project
@@ -2187,11 +2215,10 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         proj = self.get_project(name_or_id)
         if not proj:
             raise OpenStackCloudException("project does not exist")
-        try:
-            return self.manager.submit_task(
-                _tasks.CinderQuotasDelete(tenant_id=proj.id))
-        except cinder_exceptions.BadRequest:
-            raise OpenStackCloudException("cinder client call failed")
+
+        return self._volume_client.delete(
+            '/os-quota-sets/{tenant_id}'.format(tenant_id=proj.id),
+            error_message="cinder client call failed")
 
     def set_network_quotas(self, name_or_id, **kwargs):
         """ Set a network quota in a project
@@ -2207,11 +2234,11 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         if not proj:
             raise OpenStackCloudException("project does not exist")
 
-        body = {'quota': kwargs}
-        with _utils.neutron_exceptions("network client call failed"):
-            self.manager.submit_task(
-                _tasks.NeutronQuotasSet(tenant_id=proj.id,
-                                        body=body))
+        self._network_client.put(
+            '/quotas/{project_id}.json'.format(project_id=proj.id),
+            json={'quota': kwargs},
+            error_message=("Error setting Neutron's quota for "
+                           "project {0}".format(proj.id)))
 
     def get_network_quotas(self, name_or_id):
         """ Get network quotas for a project
@@ -2224,9 +2251,10 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         proj = self.get_project(name_or_id)
         if not proj:
             raise OpenStackCloudException("project does not exist")
-        with _utils.neutron_exceptions("network client call failed"):
-            return self.manager.submit_task(
-                _tasks.NeutronQuotasGet(tenant_id=proj.id))
+        return self._network_client.get(
+            '/quotas/{project_id}.json'.format(project_id=proj.id),
+            error_message=("Error fetching Neutron's quota for "
+                           "project {0}".format(proj.id)))
 
     def delete_network_quotas(self, name_or_id):
         """ Delete network quotas for a project
@@ -2240,9 +2268,10 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         proj = self.get_project(name_or_id)
         if not proj:
             raise OpenStackCloudException("project does not exist")
-        with _utils.neutron_exceptions("network client call failed"):
-            return self.manager.submit_task(
-                _tasks.NeutronQuotasDelete(tenant_id=proj.id))
+        self._network_client.delete(
+            '/quotas/{project_id}.json'.format(project_id=proj.id),
+            error_message=("Error deleting Neutron's quota for "
+                           "project {0}".format(proj.id)))
 
     def list_magnum_services(self):
         """List all Magnum services.
