@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -24,7 +22,7 @@ from shade import OpenStackCloudException
 from shade.tests.functional import base
 
 
-class TestUsers(base.BaseFunctionalTestCase):
+class TestUsers(base.KeystoneBaseFunctionalTestCase):
     def setUp(self):
         super(TestUsers, self).setUp()
         self.user_prefix = self.getUniqueString('user')
