@@ -24,6 +24,7 @@ import os
 import os_client_config as occ
 from requests import structures
 from requests_mock.contrib import fixture as rm_fixture
+from six.moves import urllib
 import tempfile
 
 import shade.openstackcloud
@@ -174,11 +175,14 @@ class RequestsMockTestCase(BaseTestCase):
         self.use_keystone_v3()
         self.__register_uris_called = False
 
-    def get_mock_url(self, service_type, interface, resource=None,
+    def get_mock_url(self, service_type, interface='public', resource=None,
                      append=None, base_url_append=None,
                      qs_elements=None):
         endpoint_url = self.cloud.endpoint_for(
             service_type=service_type, interface=interface)
+        # Strip trailing slashes, so as not to produce double-slashes below
+        if endpoint_url.endswith('/'):
+            endpoint_url = endpoint_url[:-1]
         to_join = [endpoint_url]
         qs = ''
         if base_url_append:
@@ -241,9 +245,10 @@ class RequestsMockTestCase(BaseTestCase):
         return project_list
 
     def _get_project_data(self, project_name=None, enabled=None,
-                          domain_id=None, description=None, v3=True):
+                          domain_id=None, description=None, v3=True,
+                          project_id=None):
         project_name = project_name or self.getUniqueString('projectName')
-        project_id = uuid.uuid4().hex
+        project_id = uuid.UUID(project_id or uuid.uuid4().hex).hex
         response = {'id': project_id, 'name': project_name}
         request = {'name': project_name}
         domain_id = (domain_id or uuid.uuid4().hex) if v3 else None
@@ -268,7 +273,7 @@ class RequestsMockTestCase(BaseTestCase):
 
     def _get_group_data(self, name=None, domain_id=None, description=None):
         group_id = uuid.uuid4().hex
-        name or self.getUniqueString('groupname')
+        name = name or self.getUniqueString('groupname')
         domain_id = uuid.UUID(domain_id or uuid.uuid4().hex).hex
         response = {'id': group_id, 'name': name, 'domain_id': domain_id}
         request = {'name': name}
@@ -396,7 +401,7 @@ class RequestsMockTestCase(BaseTestCase):
         return _RoleData(role_id, role_name, {'role': response},
                          {'role': request})
 
-    def use_keystone_v3(self):
+    def use_keystone_v3(self, catalog='catalog-v3.json'):
         self.adapter = self.useFixture(rm_fixture.Fixture())
         self.calls = []
         self._uri_registry.clear()
@@ -408,7 +413,7 @@ class RequestsMockTestCase(BaseTestCase):
                  headers={
                      'X-Subject-Token': self.getUniqueString('KeystoneToken')},
                  text=open(os.path.join(
-                     self.fixtures_directory, 'catalog-v3.json'), 'r').read()
+                     self.fixtures_directory, catalog), 'r').read()
                  )
         ])
         self._make_test_cloud(identity_api_version='3')
@@ -469,6 +474,13 @@ class RequestsMockTestCase(BaseTestCase):
         discovery_fixture = os.path.join(
             self.fixtures_directory, image_version_json)
         return dict(method='GET', uri='https://image.example.com/',
+                    status_code=300,
+                    text=open(discovery_fixture, 'r').read())
+
+    def get_designate_discovery_mock_dict(self):
+        discovery_fixture = os.path.join(
+            self.fixtures_directory, "dns.json")
+        return dict(method='GET', uri="https://dns.example.com/",
                     text=open(discovery_fixture, 'r').read())
 
     def use_glance(self, image_version_json='image-version.json'):
@@ -480,6 +492,15 @@ class RequestsMockTestCase(BaseTestCase):
         self.__do_register_uris([
             self.get_glance_discovery_mock_dict(image_version_json)])
 
+    def use_designate(self):
+        # NOTE(slaweq): This method is only meant to be used in "setUp"
+        # where the ordering of the url being registered is tightly controlled
+        # if the functionality of .use_designate is meant to be used during an
+        # actual test case, use .get_designate_discovery_mock and apply to the
+        # right location in the mock_uris when calling .register_uris
+        self.__do_register_uris([
+            self.get_designate_discovery_mock_dict()])
+
     def register_uris(self, uri_mock_list=None):
         """Mock a list of URIs and responses via requests mock.
 
@@ -567,9 +588,27 @@ class RequestsMockTestCase(BaseTestCase):
             if stop_after and x > stop_after:
                 break
 
+            call_uri_parts = urllib.parse.urlparse(call['url'])
+            history_uri_parts = urllib.parse.urlparse(history.url)
             self.assertEqual(
-                (call['method'], call['url']), (history.method, history.url),
-                'REST mismatch on call {index}'.format(index=x))
+                (call['method'], call_uri_parts.scheme, call_uri_parts.netloc,
+                 call_uri_parts.path, call_uri_parts.params,
+                 urllib.parse.parse_qs(call_uri_parts.query)),
+                (history.method, history_uri_parts.scheme,
+                 history_uri_parts.netloc, history_uri_parts.path,
+                 history_uri_parts.params,
+                 urllib.parse.parse_qs(history_uri_parts.query)),
+                ('REST mismatch on call %(index)d. Expected %(call)r. '
+                 'Got %(history)r). '
+                 'NOTE: query string order differences wont cause mismatch' %
+                 {
+                     'index': x,
+                     'call': '{method} {url}'.format(method=call['method'],
+                                                     url=call['url']),
+                     'history': '{method} {url}'.format(
+                         method=history.method,
+                         url=history.url)})
+            )
             if 'json' in call:
                 self.assertEqual(
                     call['json'], history.json(),
